# -*- coding: Latin-1 -*-
# pylint: disable=too-many-lines
#
# This file created Sept 14 2010 by Jim Kornelsen
#
# 13-Nov-10 JDK  Updated French translations for 1.0 release.
# 01-Apr-11 JDK  Added Spanish.  Localize dynamic labels and status messages.
# 22-Apr-11 JDK  Strings need to be unicode to handle accents correctly.
# 28-Oct-11 JDK  Finished Spanish and French for 1.2 release.
# 27-Feb-13 JDK  Can't modify dict in loop in python 3.3
# 11-Mar-13 JDK  Remove "Default" (use underlying name "Standard" instead).
# 08-Mar-17 JDK  Spanish and French for 3.0 release.

"""
Handle localization of messages into French and Spanish.
Also gets the system locale and list of locales.

To maintain the translations dictionary, here is one approach:
1. Using Vim, change each entry in the dictionary into "eng" | "fr"
   Save as type .csv
2. Open in OOo Calc with | as delimiter.
   Sort by English, and delete duplicates.
3. Use Vim to change back into dictionary entries.

To verify the strings that should go in this file, see
build/generating_code/read_error_messages.pl.

This module exports:
    theLocale
"""
from __future__ import unicode_literals
import logging

from lingt.utils import util

logger = logging.getLogger("lingt.utils.locale")

# The system locale (instantiated after class definition).
theLocale = None

class Locale:
    """Call loadUnoObjs() before using getText()."""

    def __init__(self):
        self.unoObjs = None
        self.code = None  # two-letter ISO language code

    def loadUnoObjs(self, genericUnoObjs):
        """Initialize and get current OpenOffice locale."""
        if self.unoObjs:
            # Already loaded.
            return theLocale
        self.unoObjs = genericUnoObjs

        ## Get locale setting

        configProvider = self.unoObjs.smgr.createInstanceWithContext(
            "com.sun.star.configuration.ConfigurationProvider",
            self.unoObjs.ctx)
        args = (
            # trailing comma is required to make a tuple
            util.createProp("nodepath", "/org.openoffice.Setup/L10N"),
        )
        settings = configProvider.createInstanceWithArguments(
            "com.sun.star.configuration.ConfigurationAccess", args)
        OOLang = settings.getByName("ooLocale")
        self.code = OOLang[:2]  # grab first two characters
        logger.debug("locale = %s", self.code)

        ## Make the English key values case insensitive

        translationsLower = dict()
        for en in self.translations:
            translationsLower[en.lower()] = self.translations[en]
        self.translations.update(translationsLower)
        return theLocale

    def getText(self, message_en):
        """Return L10N value.  If no translation is found for the current
        locale, returns the English message.
        """
        if message_en is None:
            return ""
        message_en = str(message_en)
        if self.code == "en":
            return message_en
        key = message_en.lower()
        if key in self.translations:
            phrase_translations = self.translations.get(key)
            message_other = phrase_translations.get(self.code)
            if message_other is not None and message_other != "":
                return message_other
        return message_en

    # ISO language codes used in struct com.sun.star.lang.Locale
    LANG_CODES = {
        'aa' : "Afar",
        'ab' : "Abkhazian",
        'af' : "Afrikaans",
        'am' : "Amharic",
        'ar' : "Arabic",
        'as' : "Assamese",
        'ay' : "Aymara",
        'az' : "Azerbaijani",
        'ba' : "Bashkir",
        'be' : "Byelorussian",
        'bg' : "Bulgarian",
        'bh' : "Bihari",
        'bi' : "Bislama",
        'bn' : "Bengali",
        'bo' : "Tibetan",
        'br' : "Breton",
        'ca' : "Catalan",
        'co' : "Corsican",
        'cs' : "Czech",
        'cy' : "Welsh",
        'da' : "Danish",
        'de' : "German",
        'dz' : "Bhutani",
        'el' : "Greek",
        'en' : "English",
        'eo' : "Esperanto",
        'es' : "Spanish",
        'et' : "Estonian",
        'eu' : "Basque",
        'fa' : "Persian",
        'fi' : "Finnish",
        'fj' : "Fiji",
        'fo' : "Faroese",
        'fr' : "French",
        'fy' : "Frisian",
        'ga' : "Irish",
        'gd' : "Scots Gaelic",
        'gl' : "Galician",
        'gn' : "Guarani",
        'gu' : "Gujarati",
        'ha' : "Hausa",
        'he' : "Hebrew",
        'hi' : "Hindi",
        'hr' : "Croatian",
        'hu' : "Hungarian",
        'hy' : "Armenian",
        'ia' : "Interlingua",
        'id' : "Indonesian",
        'ie' : "Interlingue",
        'ik' : "Inupiak",
        'is' : "Icelandic",
        'it' : "Italian",
        'iu' : "Inuktitut",
        'ja' : "Japanese",
        'jw' : "Javanese",
        'ka' : "Georgian",
        'kk' : "Kazakh",
        'kl' : "Greenlandic",
        'km' : "Cambodian",
        'kn' : "Kannada",
        'ko' : "Korean",
        'ks' : "Kashmiri",
        'ku' : "Kurdish",
        'ky' : "Kirghiz",
        'la' : "Latin",
        'ln' : "Lingala",
        'lo' : "Laothian",
        'lt' : "Lithuanian",
        'lv' : "Latvian",
        'mg' : "Malagasy",
        'mi' : "Maori",
        'mk' : "Macedonian",
        'ml' : "Malayalam",
        'mn' : "Mongolian",
        'mo' : "Moldavian",
        'mr' : "Marathi",
        'ms' : "Malay",
        'mt' : "Maltese",
        'my' : "Burmese",
        'na' : "Nauru",
        'ne' : "Nepali",
        'nl' : "Dutch",
        'no' : "Norwegian",
        'oc' : "Occitan",
        'om' : "Oromo (Afan)",
        'or' : "Oriya",
        'pa' : "Punjabi",
        'pl' : "Polish",
        'ps' : "Pashto, Pushto",
        'pt' : "Portuguese",
        'qu' : "Quechua",
        'rm' : "Rhaeto-Romance",
        'rn' : "Kirundi",
        'ro' : "Romanian",
        'ru' : "Russian",
        'rw' : "Kinyarwanda",
        'sa' : "Sanskrit",
        'sd' : "Sindhi",
        'sg' : "Sangho",
        'sh' : "Serbo-Croatian",
        'si' : "Sinhalese",
        'sk' : "Slovak",
        'sl' : "Slovenian",
        'sm' : "Samoan",
        'sn' : "Shona",
        'so' : "Somali",
        'sq' : "Albanian",
        'sr' : "Serbian",
        'ss' : "Siswati",
        'st' : "Sesotho",
        'su' : "Sundanese",
        'sv' : "Swedish",
        'sw' : "Swahili",
        'ta' : "Tamil",
        'te' : "Telugu",
        'tg' : "Tajik",
        'th' : "Thai",
        'ti' : "Tigrinya",
        'tk' : "Turkmen",
        'tl' : "Tagalog",
        'tn' : "Setswana",
        'to' : "Tonga",
        'tr' : "Turkish",
        'ts' : "Tsonga",
        'tt' : "Tatar",
        'tw' : "Twi",
        'ug' : "Uighur",
        'uk' : "Ukrainian",
        'ur' : "Urdu",
        'uz' : "Uzbek",
        'vi' : "Vietnamese",
        'vo' : "Volapuk",
        'wo' : "Wolof",
        'xh' : "Xhosa",
        'yi' : "Yiddish",
        'yo' : "Yoruba",
        'za' : "Zhuang",
        'zh' : "Chinese",
        'zu' : "Zulu"}

    def getLocaleList(self):
        """Returns a list of tuples with locale description and locale obj.
        This only shows a few locales set by the user, not a big list.
        """
        oLingu = self.unoObjs.smgr.createInstanceWithContext(
            "com.sun.star.linguistic2.LinguServiceManager",
            self.unoObjs.ctx)

        # css.linguistic2.Thesaurus, SpellChecker and Proofreader are allowed
        localeObjs = oLingu.getAvailableLocales(
            "com.sun.star.linguistic2.Thesaurus")

        descList = []
        for localeObj in localeObjs:
            desc = "%s (%s)" % (
                Locale.LANG_CODES[localeObj.Language], localeObj.Country)
            descList.append((desc, localeObj))
        return descList

    translations = {

        ## Dynamic labels in dialogs

        "Back to Settings" : {
            'es' :
            "Volver a la configuracin",
            'fr' :
            "Atteindre la configuration",
        },
        "Column" : {
            'es' :
            "Columna",
            'fr' :
            "Colonne",
        },
        "Bulk Conversion" : {
            'es' :
            "Conversin masiva",
            'fr' :
            "Conversion en bloc",
        },
        "Get Phonology Examples" : {
            'es' :
            "Obtener ejemplos de fonologa",
            'fr' :
            "Obtenir des exemples de phonologie",
        },
        "Get Interlinear Grammar Examples" : {
            'es' :
            "Obtener ejemplos de gramtica",
            'fr' :
            "Obtenir des exemples de grammaire",
        },
        "Get words" : {
            'es' :
            "Obtener palabras",
            'fr' :
            "Obtenir mots",
        },
        "Go to Practice" : {
            'es' :
            "Ir a la prctica",
            'fr' :
            "Atteindre exercices",
        },
        "Make Empty List" : {
            'es' :
            "Hacer una lista vaca",
            'fr' :
            "Crer liste vide",
        },
        "Make List" : {
            'es' :
            "Hacer una lista",
            'fr' :
            "Crer liste",
        },
        "Replace with Example" : {
            'es' :
            "Reemplazar con ejemplo",
            'fr' :
            "Remplacer par exemple",
        },
        "Replace All" : {
            'es' :
            "Reemplazar todo",
            'fr' :
            "Remplacer tout",
        },
        "Script Practice" : {
            'es' :
            "Practica de script",
            'fr' :
            "Exercices d'criture",
        },
        "Script Practice - Settings" : {
            'es' :
            "Practica de script - Configuracin",
            'fr' :
            "Exercices d'criture - Configuration",
        },
        "Searched by %s but did not find anything." : {
            'es' :
            "Se ha buscado por %s pero no ha encontrado nada.",
            'fr' :
            "Recherche selon %s n'a trouv aucun rsultat.",
        },
        "Spelling" : {
            'es' :
            "Ortografa",
            'fr' :
            "Vrification d'orthographe",
        },
        "Testing" : {
            'es' :
            "Probando",
            'fr' :
            "Test",
        },
        "Update Example" : {
            'es' :
            "Actualizar el ejemplo",
            'fr' :
            "Mettre l'exemple  jour",
        },
        "Update All" : {
            'es' :
            "Actualizar todos",
            'fr' :
            "Tout mettre  jour",
        },
        "Word List and Spelling" : {
            'es' :
            "Lista de palabras y ortografa",
            'fr' :
            "Liste de mots et orthographe",
        },

        ## Localized text values

        "(none)" : {
            'es' :
            "(ninguno)",
            'fr' :
            "(aucun)",
        },
        "(cannot make word)" : {
            'es' :
            "(no puede hacer la palabra)",
            'fr' :
            "(impossible de crer mot)",
        },
        "(Fallback Font)" : {
            'es' :
            "(Fuente de Reserva)",
            'fr' :
            "(Police de Remplacement)",
        },
        "(no words found)" : {
            'es' :
            "(no hay palabras encontradas)",
            'fr' :
            "(aucun mot trouv)",
        },
        "(No data)" : {
            'es' :
            "(No hay datos)",
            'fr' :
            "(Aucune donne)",
        },
        "Whole Document" : {
            'es' :
            "Documento completo",
            'fr' :
            "Document entier",
        },

        ## Status messages for ProgressBar

        "Converting..." : {
            'es' :
            "Convirtiendo...",
            'fr' :
            "Conversion en cours...",
        },
        "Finding text..." : {
            'es' :
            "Buscando texto...",
            'fr' :
            "Recherche de texte...",
        },
        "Generating List..." : {
            'es' :
            "Generando lista...",
            'fr' :
            "Cration de liste...",
        },
        "Getting data..." : {
            'es' :
            "Obteniendo datos...",
            'fr' :
            "Lobtention des donnes...",
        },
        "Reading..." : {
            'es' :
            "Leyendo...",
            'fr' :
            "",
        },
        "Reading files..." : {
            'es' :
            "Leyendo los archivos...",
            'fr' :
            "Lecture des fichiers...",
        },
        "Saving file..." : {
            'es' :
            "Guardando archivo...",
            'fr' :
            "Enregistrement de fichier...",
        },
        "Searching for occurrences..." : {
            'es' :
            "Buscando ocurrencias...",
            'fr' :
            "Recherche des occurrences...",
        },
        "Sorting..." : {
            'es' :
            "Ordenando...",
            'fr' :
            "Triage...",
        },
        "Loading data..." : {
            'es' :
            "Cargando datos...",
            'fr' :
            "Chargement des donnes...",
        },

        ## Error messages

        "%s is already in the list." : {
            'es' :
            "%s ya est en la lista.",
            'fr' :
            "%s est dj dans la liste.",
        },
        "Add '%s' as a new abbreviation?" : {
            'es' :
            "Agregar '%s' como una abreviatura de nuevo?",
            'fr' :
            "Ajouter '%s' comme nouvelle abrviation?",
        },
        "Cannot be in a header or footer." : {
            'es' :
            "No puede ser en un encabezado o un pie de pgina.",
            'fr' :
            "Interdit dans un en-tte ou pied de page.",
        },
        "Cannot be inside a table or frame." : {
            'es' :
            "No se puede estar dentro de una tabla o un marco.",
            'fr' :
            "Interdit dans un tableau ou cadre",
        },
        "Cannot find file %s" : {
            'es' :
            "No se puede encontrar el archivo %s",
            'fr' :
            "Impossible de trouver le fichier %s",
        },
        "Cannot insert text here." : {
            'es' :
            "No se puede insertar texto aqu.",
            'fr' :
            "Impossible d'insrer texte ici.",
        },
        "Character style '%s' is missing" : {
            'es' :
            "No se encuentra el estilo de carcter '%s'",
            'fr' :
            "Style de caractre '%s' introuvable",
        },
        "Column width is not a number." : {
            'es' :
            "El ancho de columna no es un nmero.",
            'fr' :
            "La largeur de colonne n'est pas un nombre.",
        },
        "Could not create style '%s'." : {
            'es' :
            "No se pudo crear el estilo '%s'.",
            'fr' :
            "Impossible de crer le style '%s'.",
        },
        "Could not create temporary folder %s" : {
            'es' :
            "No se pudo crear la carpeta temporal %s.",
            'fr' :
            "Impossible de crer le dossier temporaire %s.",
        },
        "Could not find any words in '%s'.  Try changing %s%d to use a "
        "different marker, or change %s to 'WordLine%d'." : {
            'es' :
            "No se encontraron palabras en '%s'. Intente cambiar %s%d para "
            "usar un marcador diferente, o cambie %s a 'WordLine%d'.",
            'fr' :
            "Impossible de trouver des mots dans '%s'. Essayer une autre "
            "balise pour %s%d, ou modifier %s en 'WordLine%d'.",
        },
        "Could not find ref number %s" : {
            'es' :
            "No se encuentra el nmero de referencia %s",
            'fr' :
            "Numro de rfrence %s introuvable.",
        },
        "Could not find ref number %s\n\nSuggestions\n%s" : {
            'es' :
            "No se encuentra el nmero de referencia %s\n\nSugerencias\n%s",
            'fr' :
            "Numro de rfrence %s introuvable.\n\nSuggestions\n%s",
        },
        "Could not get AddConverter function.  Automatically adding a "
        "converter requires SEC4.0 or higher." : {
            'es' :
            "No se pudo obtener la funcin AddConverter. La "
            "adicin automtica de un convertidor requiere "
            "SEC4.0 o superior.",
            'fr' :
            "Impossible d'obtenir la fonction AddConverter. L'ajout "
            "automatique d'un convertisseur ncessite SEC4.0 ou suprieur.",
        },
        "Could not get column %d, row %d of table %s." : {
            'es' :
            "No se pudo obtener la columna %d, fila %d de la tabla%s.",
            'fr' :
            "Impossible dobtenir colonne %d, ligne %d de table %s.",
        },
        "Could not get selection string." : {
            'es' :
            "No se pudo obtener la cadena de seleccin.",
            'fr' :
            "Impossible dobtenir la chane de slection",
        },
        "Did not find any data in file %s" : {
            'es' :
            "No ha encontrado ningn dato en el archivo %s",
            'fr' :
            "Aucune donne n'a t trouve dans le fichier %s",
        },
        "Did not find any similar words." : {
            'es' :
            "No encontr algunas palabras similares.",
            'fr' :
            "On n'a trouv aucun mot similaire.",
        },
        "Did not find any words for the list." : {
            'es' :
            "No encontr algunas palabras para la lista.",
            'fr' :
            "On n'a trouv aucun mot pour la liste.",
        },
        "Did not find anything in column %s." : {
            'es' :
            "No encontr nada en la columna %s.",
            'fr' :
            "On n'a rien trouv dans colonne %s.",
        },
        "Did not find scope of change." : {
            'es' :
            "No ha encontrado el mbito del cambio.",
            'fr' :
            "L'tendue de changement n'a pas t trouve.",
        },
        "EncConverters does not seem to be installed properly." : {
            'es' :
            "EncConverters no parece que se haya instalado correctamente.",
            'fr' :
            "EncConverters semble tre mal install",
        },
        "Error parsing %s user variable.  Please go to Insert -> Field -> "
        "More Fields and fix the problem." : {
            'es' :
            "Error al analizar %s variable de usuario.  Por favor, vaya a "
            "Insertar -> Campos y solucionar el problema.",
            'fr' :
            "Erreur en analysant la variable utilisateur %s.  Veuillez "
            "atteindre Insertion -> Champs pour rsoudre le problme.",
        },
        "Error reading file %s" : {
            'es' :
            "Error al leer el archivo %s",
            'fr' :
            "Erreur en lisant le fichier %s",
        },
        "Error reading file %s\n\n%s" : {
            'es' :
            "Error al leer el archivo %s\n\n%s",
            'fr' :
            "Erreur en lisant le fichier %s\n\n%s",
        },
        "Error saving %s" : {
            'es' :
            "Error al guardar %s",
            'fr' :
            "Erreur denregistrement de %s",
        },
        "Error showing dialog: No %s control." : {
            'es' :
            "Error al mostrar el dilogo: No %s control.",
            'fr' :
            "Erreur daffichage du dialogue: Aucun contrle %s",
        },
        "Error with file: %s" : {
            'es' :
            "Error con el archivo: %s",
            'fr' :
            "Erreur de fichier : %s",
        },
        "Error reading spreadsheet." : {
            'es' :
            "Error al leer la hoja de clculo",
            'fr' :
            "Erreur de lecture de classeur",
        },
        "Error reading the list." : {
            'es' :
            "Error al leer la lista.",
            'fr' :
            "Erreur de lecture de liste.",
        },
        "Error reading the list.\n\n%s" : {
            'es' :
            "Error al leer la lista.\n\n%s",
            'fr' :
            "Erreur de lecture de liste.\n\n%s",
        },
        "Error writing to spreadsheet." : {
            'es' :
            "Error al escribir al hoja de clculo.",
            'fr' :
            "Erreur d'criture de classeur.",
        },
        "Error: Could not create dialog." : {
            'es' :
            "Error: No se pudo crear el dilogo.",
            'fr' :
            "Erreur : Impossible de crer dialogue.",
        },
        "Error: EncConverters returned %d%s." : {
            'es' :
            "Error: EncConverters devolvi %d%s.",
            'fr' :
            "Erreur: EncConverters a rpondu %d%s.",
        },
        "Expected frame margin parameter." : {
            'es' :
            "Parmetro de margen de marco esperado.",
            'fr' :
            "Paramtre de marge de cadre attendu.",
        },
        "Expected styleFonts to be set." : {
            'es' :
            "Se espera que se establezcan styleFonts.",
            'fr' :
            "Paramtre de styleFonts attendu.",
        },
        "Failed to encode string properly." : {
            'es' :
            "No pudo codificar correctamente la cadena.",
            'fr' :
            "Impossible d'encoder correctement la chane.",
        },
        "Failed to go to text range." : {
            'es' :
            "No pudo ir al rango de texto.",
            'fr' :
            "Impossible datteindre la plage de texte.",
        },
        "File does not seem to be from Toolbox or FieldWorks: %s" : {
            'es' :
            "El archivo no parece ser del Toolbox o Fieldworks: %s",
            'fr' :
            "Il semble que ce fichier n'a pas t cr par Toolbox ou "
            "FieldWorks: %s",
        },
        "File is already in the list." : {
            'es' :
            "El archivo ya est en la lista.",
            'fr' :
            "Le fichier est dj dans la liste.",
        },
        "First press Copy." : {
            'es' :
            "Primero presione Copiar.",
            'fr' :
            "Appuyer d'abord sur Copier.",
        },
        "Found %d similar words." : {
            'es' :
            "Encontrado %d palabras similares.",
            'fr' :
            "%d mots similaires trouvs.",
        },
        "Found %d words." : {
            'es' :
            "Encontrado %d palabras.",
            'fr' :
            "%d mots trouvs.",
        },
        "Found %d paragraphs and made %d change%s." : {
            'es' :
            "Ha encontrado %d prrafos y hizo %d cambio%s.",
            'fr' :
            "%d paragraphes trouvs et %d changement%s faits.",
        },
        "Found a ref number, but it must be in an outer table in order to "
        "be updated." : {
            'es' :
            "Ha encontrado un nmero de referencia, pero debe estar en una "
            "tabla de exterior para ser actualizados.",
            'fr' :
            "N de rf.  trouv, mais pour l'actualier il doit tre dans un "
            "cadre exterieur",
        },
        "Frame style '%s' is missing" : {
            'es' :
            "No se encuentra el estilo del marco '%s'",
            'fr' :
            "Style de cadre '%s' introuvable",
        },
        "If you want to use LIFT data, then first specify a LIFT file "
        "exported from FieldWorks." : {
            'es' :
            "Si desea utilizar los datos LIFT, en primer lugar especificar "
            "un archivo LIFT exportados de Fieldworks.",
            'fr' :
            "Pour utiliser des donnes LIFT il faut spcifier un fichier "
            "LIFT export de FieldWorks.",
        },
        "Library error: %s." : {
            'es' :
            "Error de rutinas: %s.",
            'fr' :
            "Erreur de logiciel: %s.",
        },
        "Made %d change%s to %d file%s." : {
            'es' :
            "Hecho %d cambio%s a %d archivo %s.",
            'fr' :
            "%d changement%s faits dans %d fichier%s.",
        },
        "Made %d change%s." : {
            'es' :
            "Hizo %d cambio%s.",
            'fr' :
            "%d changement%s faits.",
        },
        "Made %d correction%s." : {
            'es' :
            "Hizo %d correccione%s.",
            'fr' :
            "On a fait %d correction%s.",
        },
        "Made list of %d words." : {
            'es' :
            "Hizo una lista de %d palabras.",
            'fr' :
            "On a cr une liste de %d mots.",
        },
        "Make this change?" : {
            'es' :
            "Hacer este cambio?",
            'fr' :
            "Modifier ceci?",
        },
        "Make this change?  (%s -> %s)" : {
            'es' :
            "Hacer este cambio?  (%s -> %s)",
            'fr' :
            "Modifier ceci?  (%s -> %s)",
        },
        "Missed word '%s'.  Keep going?" : {
            'es' :
            "Hubo un problema con la palabra '%s'.  Seguir adelante?",
            'fr' :
            "Un problme en le mot '%s'.  Continuer?",
        },
        "No conversion result." : {
            'es' :
            "No hay resultado de conversin.",
            'fr' :
            "Aucun rsultat de conversion.",
        },
        "No changes, but modified style of %d paragraph%s." : {
            'es' :
            "No hubo cambios, pero el estilo de %d prrafo%s se ha "
            "modificado.",
            'fr' :
            "Pas de changements, mais le style de %d paragraphe%s a t "
            "chang.",
        },
        "No changes." : {
            'es' :
            "No hubo cambios.",
            'fr' :
            "Pas de changements.",
        },
        "No converter was specified." : {
            'es' :
            "No convertidor se ha especificado.",
            'fr' :
            "Aucun convertisseur spcifi.",
        },
        "No data found." : {
            'es' :
            "No se encontraron datos",
            'fr' :
            "Aucune donne trouve.",
        },
        "No locale was specified." : {
            'es' :
            "Un locale no se ha especificado",
            'fr' :
            "Aucuns paramtres rgionaux spcifis.",
        },
        "No more existing examples found." : {
            'es' :
            "No se ha encontrado ms ejemplos existentes",
            'fr' :
            "Il n'y a plus d'exemples trouvs.",
        },
        "No more possible abbreviations found." : {
            'es' :
            "No se ha encontrado ms abreviaturas posibles",
            'fr' :
            "On ne trouve plus des abrviations possibles.",
        },
        "No more reference numbers found." : {
            'es' :
            "No se ha encontrado ms nmeros de referencia",
            'fr' :
            "On ne trouve plus des numros de rfrence.",
        },
        "No more reference numbers found.\n Make sure to type # in front." : {
            'es' :
            "No se encontraron ms nmeros de referencia. \n Asegrese de "
            "escribir # delante.",
            'fr' :
            "On ne trouve plus des numros de rfrence. Il faut vrifier "
            "le # devant des numros.",
        },
        "No scope was specified." : {
            'es' :
            "No se especific ningn mbito.",
            'fr' :
            "Aucune tendue ntait spcifie",
        },
        "No SF markers were specified.  Continue anyway?" : {
            'es' :
            "Ningn marcadores SFM fueron especificados.  Desea continuar?",
            'fr' :
            "Aucune balise SFM spcifie.  Continuer quand mme?",
        },
        "No spreadsheet is open." : {
            'es' :
            "No hay ninguna hoja de clculo abierto.",
            'fr' :
            "Aucun classeur est ouvert.",
        },
        "No task was specified." : {
            'es' :
            "No se especific ninguna tarea.",
            'fr' :
            "Aucune tche ntait spcifie",
        },
        "No writing systems found." : {
            'es' :
            "No se encontraron sistemas de escritura.",
            'fr' :
            "Aucune systmes d'criture trouve.",
        },
        "No Xpath expressions were specified.  Continue anyway?" : {
            'es' :
            "Ningn expresiones XPath fueron especificadas.  "
            "Desea continuar?",
            'fr' :
            "Aucune expression Xpath spcifie.  Continuer quand mme?",
        },
        "No text is selected." : {
            'es' :
            "No hay texto seleccionado.",
            'fr' :
            "Aucun texte slectionn.",
        },
        "Paragraph style '%s' is missing" : {
            'es' :
            "No se encuentra el estilo de prrafo '%s'",
            'fr' :
            "Style de paragraphe '%s' introuvable",
        },
        "Please add a file to get words." : {
            'es' :
            "Por favor, aada un archivo para obtener las palabras.",
            'fr' :
            "Veuillez ajouter un fichier duquel on peut obtenir des mots.",
        },
        "Please add files to scan." : {
            'es' :
            "Por favor, agregue archivos para escanear.",
            'fr' :
            "Veuillez ajouter des fichiers  analyser.",
        },
        "Please do not select individual table cells." : {
            'es' :
            "Por favor, no seleccione las celdas individuales de la tabla.",
            'fr' :
            "Veuillez ne pas choisir des cellules individuelles.",
        },
        "Please enter a number for max length." : {
            'es' :
            "Por favor, introduzca un nmero para la longitud mxima.",
            'fr' :
            "Veuillez entrer la longueur maximum.",
        },
        "Please enter a ref number." : {
            'es' :
            "Por favor, introduzca un nmero de referencia.",
            'fr' :
            "Veuillez entrer un numro de rfrence.",
        },
        "Please enter a ref number.\n\nSuggestions\n%s" : {
            'es' :
            "Por favor, introduzca un nmero de referencia."
            "\n\nSugerencias\n%s",
            'fr' :
            "Veuillez entrer un numro de rfrence.\n\nSuggestions\n%s",
        },
        "Please enter a value for column width." : {
            'es' :
            "Por favor, introduzca un valor para el ancho de la columna.",
            'fr' :
            "Veuillez entrer la largeur de colonne.",
        },
        "Please go to Grammar Settings and specify a file." : {
            'es' :
            "Por favor, vaya a la Configuracin de gramtica y especifique "
            "un archivo.",
            'fr' :
            "Veuillez choisir un fichier dans Configuration de grammaire.",
        },
        "Please go to Phonology Settings and specify a file." : {
            'es' :
            "Por favor, vaya a la Configuracin de fonologa y especifique "
            "un archivo.",
            'fr' :
            "Veuillez spcifier un fichier dans Configuration de phonologie.",
        },
        "Please load a word list by clicking on the Files... button.  When "
        "file settings are finished, click Get words." : {
            'es' :
            "Por favor, cargue una lista de palabras haciendo clic en el "
            "botn Archivos.  Cuando la configuracin de archivo se haya "
            "terminado, haga clic en Obtener palabras.",
            'fr' :
            "Veuillez charger une liste de mots en cliquant sur Fichiers...  "
            "Aprs avoir fait la configuration de fichier",
        },
        "Please save the current document first." : {
            'es' :
            "Por favor, primero guarde el documento actual.",
            'fr' :
            "Veuillez enregistrer d'abord le document actuel.",
        },
        "Please save the spreadsheet first." : {
            'es' :
            "Por favor, primero guarde la hoja de clculo.",
            'fr' :
            "Veuillez d'abord enregistrer le classeur",
        },
        "Please select %s in the list." : {
            'es' :
            "Por favor, seleccione %s en la lista.",
            'fr' :
            "Veuillez choisir %s dans la liste.",
        },
        "Please select a converter." : {
            'es' :
            "Por favor, seleccione un convertidor.",
            'fr' :
            "Veuillez choisir un convertisseur.",
        },
        "Please select a file in the list." : {
            'es' :
            "Por favor, seleccione un archivo en la lista.",
            'fr' :
            "Veuillez choisir un fichier dans la liste.",
        },
        "Please select a language name." : {
            'es' :
            "Por favor, seleccione un nombre de idioma.",
            'fr' :
            "Veuillez slectionner un nom de langue.",
        },
        "Please select a scope character style." : {
            'es' :
            "Por favor, seleccione un estilo de carcter mbito.",
            'fr' :
            "Veuillez choisir un style de caractre pour l'tendue.",
        },
        "Please select a scope font." : {
            'es' :
            "Por favor, seleccione una fuente mbito.",
            'fr' :
            "Veuillez choisir une police pour l'tendue.",
        },
        "Please select a paragraph style." : {
            'es' :
            "Por favor, seleccione un estilo de prrafo.",
            'fr' :
            "Veuillez choisir un style de paragraphe.",
        },
        "Please select a scope paragraph style." : {
            'es' :
            "Por favor, seleccione un estilo de prrafo mbito.",
            'fr' :
            "Veuillez choisir un style de paragraphe pour l'tendue.",
        },
        "Please select a script." : {
            'es' :
            "Por favor, seleccione un script.",
            'fr' :
            "Veuillez choisir un criture.",
        },
        "Please select a target font." : {
            'es' :
            "Por favor, seleccione una fuente destino.",
            'fr' :
            "Veuillez choisir une police cible.",
        },
        "Please select a target style." : {
            'es' :
            "Por favor, seleccione un estilo destino.",
            'fr' :
            "Veuillez choisir un style cible.",
        },
        "Please select an abbreviation in the list." : {
            'es' :
            "Por favor, seleccione una abreviatura de la lista.",
            'fr' :
            "Veuillez choisir une abrviation dans la liste.",
        },
        "Please select an item in the list." : {
            'es' :
            "Por favor, seleccione un elemento de la lista.",
            'fr' :
            "Veuillez choisir un lment dans la liste.",
        },
        "Please select an output folder." : {
            'es' :
            "Por favor, seleccione una carpeta de salida.",
            'fr' :
            "Veuillez choisir un dossier cible.",
        },
        "Please select or enter something to find." : {
            'es' :
            "Por favor seleccione o escriba algo para buscar.",
            'fr' :
            "Veuillez slectionner ou saisir quelque chose  rechercher.",
        },
        "Please select the converter again." : {
            'es' :
            "Por favor, seleccione el convertidor de nuevo.",
            'fr' :
            "Veuillez choisir encore le convertisseur.",
        },
        "Please specify SFMs." : {
            'es' :
            "Por favor, especifique los SFMs.",
            'fr' :
            "Veuillez spcifier les balises (SFMs).",
        },
        "Please specify a file to export." : {
            'es' :
            "Por favor, especifique un archivo para exportar.",
            'fr' :
            "Veuillez spcifier un fichier  exporter.",
        },
        "Please specify a scope." : {
            'es' :
            "Por favor, especifique un mbito.",
            'fr' :
            "Veuillez spcifier l'tendue.",
        },
        "Please specify a target." : {
            'es' :
            "Por favor, especifique un destino.",
            'fr' :
            "Veuillez spcifier un cible.",
        },
        "Please specify a word list file.  To make a new empty list, go to "
        "Word List and Spelling and then save the spreadsheet file." : {
            'es' :
            "Por favor, especifique un archivo de una lista de palabras.  "
            "Para crear una nueva lista vaca, vaya a Lista de palabras y "
            "ortografa y guarde el archivo de hoja de clculo.",
            'fr' :
            "Veuillez spcifier un fichier de liste de mots.  Pour crer une "
            "nouvelle liste vide, atteindre Liste de mots et orthographe, "
            "puis enregistrer le classeur.",
        },
        "Replaced %d example%s." : {
            'es' :
            "reemplazado %d ejemplo%s.",
            'fr' :
            "%d exemple%s a t remplas.",
        },
        "Spell check finished." : {
            'es' :
            "Spell check finished.",
            'fr' :
            "Vrification d'orthographe termine.",
        },
        "Successfully finished conversion." : {
            'es' :
            "Terminado con xito la conversin.",
            'fr' :
            "Conversion termine avec succs.",
        },
        "The cursor cannot be in a header or footer." : {
            'es' :
            "El cursor no puede estar en un encabezado o en un pie de "
            "pgina.",
            'fr' :
            "Le curseur ne peut pas se trouver dans un en-tte ou dans un "
            "pied de page.",
        },
        "The cursor cannot be inside a table or frame." : {
            'es' :
            "El cursor no puede estar dentro de una tabla o un marco.",
            'fr' :
            "Le curseur ne peut pas se trouver dans un tableau ou dans un "
            "cadre.",
        },
        "There was a problem while writing the list.\n\n%s" : {
            'es' :
            "Hubo un problema al escribir la lista.\n\n%s",
            'fr' :
            "Un problme est survenu en crivant la liste.\n\n%s",
        },
        "This document stores %s settings.  "
        "Please leave it open while using %s.  "
        "If you want to keep the settings to use again later, "
        "then save this document." : {
            'es' :
            "Este documento guarda la configuracin de %s.  "
            "Por favor, dejarlo abierto durante el uso de %s.  "
            "Si desea mantener la configuracin para utilizarlo ms "
            "adelante, guarde este documento.",
            'fr' :
            "Ce document contient la configuration de la fonction %s.  "
            "Veuillez le laisser ouvert en utilisant %s.  "
            "Pour garder la configuration afin de la rutiliser plus tard, "
            "enregistrer ce document.",
        },
        "This expression is already in the list." : {
            'es' :
            "Esta expresin est ya en la lista.",
            'fr' :
            "Cette expression est dj dans la liste.",
        },
        "This will change the case of the entire list from '%s' to '%s.' "
        "Continue?" : {
            'es' :
            "Esto cambiar el caso de la lista completa de '%s' a '%s'.  "
            "Desea continuar?",
            'fr' :
            "Ceci changera la casse de toute la liste de '%s'  '%s'.  "
            "Continuer?",
        },
        "This word was already set to correct.  Change anyway?" : {
            'es' :
            "Esta palabra ya estaba configurada para corregir. "
            "Cambiar de todos modos?",
            'fr' :
            "Ce mot est dj rgl pour corriger. Modifier quand mme?",
        },
        "To update examples, 'Outer table' must be marked in Grammar "
        "Settings." : {
            'es' :
            "'Tabla de exterior' debe estar en la Configuracin de "
            "gramtica.",
            'fr' :
            "'Cadre exterieur' doit tre dans Configuration de grammaire.",
        },
        "Too many files named like %s." : {
            'es' :
            "Demasiados archivos con el nombre %s.",
            'fr' :
            "Trop de fichiers nomms comme %s.",
        },
        "Too many temporary folders in %s." : {
            'es' :
            "Demasiadas carpetas temporales en %s.",
            'fr' :
            "Trop de dossiers temporaires dans %s.",
        },
        "Unexpected file type %s" : {
            'es' :
            "Tipo de archivo inesperado %s",
            'fr' :
            "Type de fichier %s inattendu",
        },
        "Unexpected font type %s." : {
            'es' :
            "Tipo de fuente inesperada %s.",
            'fr' :
            "Police de type %s inattendue.",
        },
        "Unexpected new value %s." : {
            'es' :
            "Nuevo valor inesperado %s.",
            'fr' :
            "Nouvelle valeur inattendue %s.",
        },
        "Unexpected value %r" : {
            'es' :
            "Valor inesperado %r.",
            'fr' :
            "Valeur inattendue %r",
        },
        "Unexpected value %s" : {
            'es' :
            "Valor inesperado %s.",
            'fr' :
            "Valeur inattendue %s",
        },
        "Unknown action command '%s'" : {
            'es' :
            "Comando de accin desconocido '%s'",
            'fr' :
            "Commande d'action inconnue '%s'",
        },
        "Unknown file type for %s" : {
            'es' :
            "Tipo de archivo desconocido para %s",
            'fr' :
            "Type de fichier inconnu pour %s",
        },
        "Unknown grabKey '%s'" : {
            'es' :
            "grabKey '%s' desconocido",
            'fr' :
            "grabKey '%s' inconnu",
        },
        "Update all examples now?  It is recommended to save a copy of your "
        "document first." : {
            'es' :
            "Actualizar todos los ejemplos ahora?  Se recomienda que "
            "primero guarde una copia de su documento.",
            'fr' :
            "Actualiser tous les exemples maintenant?  Il est conseill "
            "d'enregistrer le document d'abord.",
        },
        "Updated '%s' %d times in a row.  Keep going?" : {
            'es' :
            "Actualizado '%s' %d veces seguidas.  Seguir adelante?",
            'fr' :
            "'%s' a t actualis %d fois de suite.  Continuer?",
        },
        "Updated %d example%s." : {
            'es' :
            "Actualizado %d ejemplo%s.",
            'fr' :
            "%d exemple%s a t actualis.",
        },
        "Value %d for column width is too high." : {
            'es' :
            "El valor de %d para el ancho de columna es demasiado alta.",
            'fr' :
            "%d est trop grande comme largeur de colonne.",
        },
        "Value for column width must be more than zero." : {
            'es' :
            "El valor para el ancho de columna debe ser mayor de cero.",
            'fr' :
            "La largeur de colonne doit tre suprieure  zro.",
        },
        "'Whole Document' must be the only thing to find." : {
            'es' :
            "'Documento Completo' debe ser la nica cosa para buscar.",
            'fr' :
            "'Document entier' doit tre la seule chose  rechercher",
        },
        "Word failed to fit properly." : {
            'es' :
            "La palabra no se ajust correctamente.",
            'fr' :
            "Le mot ne rentre pas correctement.",
        },
        "You did not made any changes to the word." : {
            'es' :
            "No hiciste ningn cambio en la palabra.",
            'fr' :
            "Vous n'avez apport aucun changement au mot.",
        },
        "You did not specify anything to find.  Continue anyway?" : {
            'es' :
            "No ha especificado nada que encontrar.  Desea continuar?",
            'fr' :
            "Vous n'avez spcifier aucune chose  rechercher.  Continuer "
            "quand mme?",
        },
    }

theLocale = Locale()
